==> avidemux_2.0.40.orig.tar.gz: avidemux-2.0.40/avidemux/ADM_video/ADM_lavpp_deint.cpp <==
//
// C++ Implementation: %{MODULE}
//
// Description: 
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//  Small wrapper for lavcodec deinterlacer postprocessing


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ADM_assert.h>

#include "config.h"
#include "fourcc.h"

#include "ADM_toolkit/toolkit.hxx"
#include "ADM_editor/ADM_edit.hxx"
#include "ADM_video/ADM_genvideo.hxx"
#include "ADM_filter/video_filters.h"

#include "ADM_toolkit/filesel.h"

#include "ADM_colorspace/colorspace.h"

#include "ADM_toolkit/ADM_cpuCap.h"

#include "ADM_toolkit/ADM_debugID.h"
#define MODULE_NAME MODULE_FILTER
#include "ADM_toolkit/ADM_debug.h"

#include "ADM_lavpp_deintparam.h"

/*
{"al", "autolevels",            0, 1, 2, LEVEL_FIX},
{"lb", "linblenddeint",         1, 1, 4, LINEAR_BLEND_DEINT_FILTER},
{"li", "linipoldeint",          1, 1, 4, LINEAR_IPOL_DEINT_FILTER},
{"ci", "cubicipoldeint",        1, 1, 4, CUBIC_IPOL_DEINT_FILTER},
{"md", "mediandeint",           1, 1, 4, MEDIAN_DEINT_FILTER},
{"fd", "ffmpegdeint",           1, 1, 4, FFMPEG_DEINT_FILTER},
{"tn", "tmpnoise",              1, 7, 8, TEMP_NOISE_FILTER},

{"fq", "forcequant",            1, 0, 0, FORCE_QUANT}, *******************
{"l5", "lowpass5",              1, 1, 4, LOWPASS5_DEINT_FILTER}, *********
*/


extern uint8_t DIA_lavpp_deint(lavc_pp_param *param);

class  ADMVideoLavPPDeint:public AVDMGenericVideoStream
{

  protected:
    virtual char                    *printConf(void);
    void                            *ppcontext;
    void                            *ppmode;
    ADMImage                        *uncompressed;
    lavc_pp_param                   *_param;
    
    void                            setup( void );
    void                            cleanup( void );
  public:
    
    
    ADMVideoLavPPDeint(  AVDMGenericVideoStream *in,CONFcouple *setup);
    ~ADMVideoLavPPDeint();
    virtual uint8_t getFrameNumberNoAlloc(uint32_t frame, uint32_t *len,
                                          ADMImage *data,uint32_t *flags);
    virtual uint8_t getCoupledConf( CONFcouple **couples)           ;
    virtual uint8_t configure( AVDMGenericVideoStream *instream);
                                                        
};


static FILTER_PARAM lavppdeint_param={1,{"subname"}};
//*************************************************************
//

//*************************************************************
SCRIPT_CREATE(lavppdeint_script,ADMVideoLavPPDeint,lavppdeint_param);
BUILD_CREATE(lavppdeint_create,ADMVideoLavPPDeint);
//*************************************************************
uint8_t ADMVideoLavPPDeint::configure(AVDMGenericVideoStream *in)
{

  if(DIA_lavpp_deint(_param))
  {
    setup();
    return 1; 
  }
  return 0;        
}
 
//*************************************************************
char *ADMVideoLavPPDeint::printConf( void )
{
  static char buf[50];
        
  sprintf((char *)buf," Lavcodec PP deinterlacer autolev:%d deint:%d",_param->autolevel,_param->deintType);
  return buf;
}
//*************************************************************
ADMVideoLavPPDeint::ADMVideoLavPPDeint(  AVDMGenericVideoStream *in,CONFcouple *couples)
{

  _in=in;         
  memcpy(&_info,_in->getInfo(),sizeof(_info));    
  _info.encoding=1;  
  
  ppcontext=NULL;  
  ppmode=NULL;   
  
  _uncompressed=new ADMImage(_info.width,_info.height);
  if(couples)
  {   
    _param=NEW(lavc_pp_param);
    GET(deintType);    
    GET(autolevel);
  }      
  else
  {
    _param=NEW(lavc_pp_param);
    _param->deintType=0;
    _param->autolevel=0;
    
  } 
                 
  setup();
 
}
//*************************************************************
void ADMVideoLavPPDeint::cleanup(void)
{
  if(ppcontext)
  {
    pp_free_context(ppcontext);
    ppcontext=NULL;
  }
  if(ppmode)
  {
    pp_free_mode(ppmode);
    ppmode=NULL;
  }
  
} 
//*************************************************************
void ADMVideoLavPPDeint::setup(void)
{
  char string[1024];
  uint32_t ppCaps=0;
                
  string[0]=0;
  
        cleanup();
#if (defined( ARCH_X86)  || defined(ARCH_X86_64))
#define ADD(x,y) if( CpuCaps::has##x()) ppCaps|=PP_CPU_CAPS_##y;                
                ADD(MMX,MMX);           
                ADD(3DNOW,3DNOW);
                ADD(MMXEXT,MMX2);
#endif             
        cleanup();
#undef ADD       
#define ADD(z)  { if(string[0]) strcat(string,#z); else strcpy(string,#z);}        
               
        if(_param->autolevel) ADD(al);
        switch(_param->deintType)
        {
          case PP_BM_NONE:break;
          case PP_BM_LINEAR_BLEND: ADD(lb);break;
          case PP_BM_LINEAR_INTER: ADD(li);break;
          case PP_BM_CUBIC_INTER: ADD(ci);break;
          case PP_BM_MEDIAN_INTER: ADD(md);break;
          case PP_BM_FFMPEG_DEINT: ADD(fd);break;                             
        }        


        ppcontext=pp_get_context(_info.width, _info.height, ppCaps);           
        ppmode=pp_get_mode_by_name_and_quality(string,1);;
        
        ADM_assert(ppcontext);
        ADM_assert(ppmode);
  
} 
//*************************************************************
ADMVideoLavPPDeint::~ADMVideoLavPPDeint()
{
  cleanup();
  if(_uncompressed)
        delete _uncompressed;
  DELETE(_param);
  _uncompressed=NULL;
}

//*************************************************************
uint8_t ADMVideoLavPPDeint::getCoupledConf( CONFcouple **couples)
{
  
  *couples=new CONFcouple(2);
  CSET(deintType);    
  CSET(autolevel);
  
  return 1;
}
//*************************************************************
uint8_t ADMVideoLavPPDeint::getFrameNumberNoAlloc(uint32_t frame,
                                              uint32_t *len,
                                              ADMImage *data,
                                              uint32_t *flags)
{

 
  ADM_assert(frame<_info.nb_frames);
        // read uncompressed frame
  if(!_in->getFrameNumberNoAlloc(frame, len,_uncompressed,flags)) return 0;

  
  //
  uint8_t *iBuff[3],*oBuff[3];
  int strideTab[3],strideTab2[3];                 
                                                                
  oBuff[0]=YPLANE(data);
  oBuff[1]=UPLANE(data);
  oBuff[2]=VPLANE(data);
                                
  iBuff[0]=YPLANE(_uncompressed);
  iBuff[1]=UPLANE(_uncompressed);
  iBuff[2]=VPLANE(_uncompressed);
                                
                                
  strideTab[0]=strideTab2[0]=_info.width;
  strideTab[1]=strideTab2[1]=_info.width>>1;
  strideTab[2]=strideTab2[2]=_info.width>>1;
        
  int type;
  if(_uncompressed->flags&AVI_KEY_FRAME)
    type=1;
  else if(_uncompressed->flags & AVI_B_FRAME)
    type=3;
  else
    type=2;
  pp_postprocess(
        iBuff,
        strideTab,
        oBuff,
        strideTab2,
        _info.width,
        _info.height,
        NULL,
        0,
        ppmode,
        ppcontext,
        type); // I ?
                                
  
  data->copyInfo(_uncompressed);
  return 1;
}

==> avidemux_2.3.0.tar.gz: avidemux_2.3.0/avidemux/ADM_video/ADM_lavpp_deint.cpp <==
//
// C++ Implementation: %{MODULE}
//
// Description: 
//
//
// Author: %{AUTHOR} <%{EMAIL}>, (C) %{YEAR}
//
// Copyright: See COPYING file that comes with this distribution
//
//  Small wrapper for lavcodec deinterlacer postprocessing


/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ADM_assert.h>

#include "config.h"
#include "fourcc.h"

#include "ADM_toolkit/toolkit.hxx"
#include "ADM_editor/ADM_edit.hxx"
#include "ADM_video/ADM_genvideo.hxx"
#include "ADM_filter/video_filters.h"

#include "ADM_toolkit/filesel.h"

#include "ADM_colorspace/colorspace.h"

#include "ADM_toolkit/ADM_cpuCap.h"

#include "ADM_toolkit/ADM_debugID.h"
#define MODULE_NAME MODULE_FILTER
#include "ADM_toolkit/ADM_debug.h"

#include "ADM_lavpp_deintparam.h"

/*
{"al", "autolevels",            0, 1, 2, LEVEL_FIX},
{"lb", "linblenddeint",         1, 1, 4, LINEAR_BLEND_DEINT_FILTER},
{"li", "linipoldeint",          1, 1, 4, LINEAR_IPOL_DEINT_FILTER},
{"ci", "cubicipoldeint",        1, 1, 4, CUBIC_IPOL_DEINT_FILTER},
{"md", "mediandeint",           1, 1, 4, MEDIAN_DEINT_FILTER},
{"fd", "ffmpegdeint",           1, 1, 4, FFMPEG_DEINT_FILTER},
{"tn", "tmpnoise",              1, 7, 8, TEMP_NOISE_FILTER},

{"fq", "forcequant",            1, 0, 0, FORCE_QUANT}, *******************
{"l5", "lowpass5",              1, 1, 4, LOWPASS5_DEINT_FILTER}, *********
*/


extern uint8_t DIA_lavpp_deint(lavc_pp_param *param);

class  ADMVideoLavPPDeint:public AVDMGenericVideoStream
{

  protected:
    virtual char                    *printConf(void);
    void                            *ppcontext;
    void                            *ppmode;
    ADMImage                        *uncompressed;
    lavc_pp_param                   *_param;
    
    void                            setup( void );
    void                            cleanup( void );
  public:
    
    
    ADMVideoLavPPDeint(  AVDMGenericVideoStream *in,CONFcouple *setup);
    ~ADMVideoLavPPDeint();
    virtual uint8_t getFrameNumberNoAlloc(uint32_t frame, uint32_t *len,
                                          ADMImage *data,uint32_t *flags);
    virtual uint8_t getCoupledConf( CONFcouple **couples)           ;
    virtual uint8_t configure( AVDMGenericVideoStream *instream);
                                                        
};


static FILTER_PARAM lavppdeint_param={2,{"deintType","autolevel"}};
//*************************************************************
//

//*************************************************************
SCRIPT_CREATE(lavppdeint_script,ADMVideoLavPPDeint,lavppdeint_param);
BUILD_CREATE(lavppdeint_create,ADMVideoLavPPDeint);
//*************************************************************
uint8_t ADMVideoLavPPDeint::configure(AVDMGenericVideoStream *in)
{

  if(DIA_lavpp_deint(_param))
  {
    setup();
    return 1; 
  }
  return 0;        
}
 
//*************************************************************
char *ADMVideoLavPPDeint::printConf( void )
{
  static char buf[50];
        
  sprintf((char *)buf," Lavcodec PP deinterlacer autolev:%d deint:%d",_param->autolevel,_param->deintType);
  return buf;
}
//*************************************************************
ADMVideoLavPPDeint::ADMVideoLavPPDeint(  AVDMGenericVideoStream *in,CONFcouple *couples)
{

  _in=in;         
  memcpy(&_info,_in->getInfo(),sizeof(_info));    
  _info.encoding=1;  
  
  ppcontext=NULL;  
  ppmode=NULL;   
  
  _uncompressed=new ADMImage(_info.width,_info.height);
  if(couples)
  {   
    _param=NEW(lavc_pp_param);
    GET(deintType);    
    GET(autolevel);
  }      
  else
  {
    _param=NEW(lavc_pp_param);
    _param->deintType=0;
    _param->autolevel=0;
    
  } 
                 
  setup();
 
}
//*************************************************************
void ADMVideoLavPPDeint::cleanup(void)
{
  if(ppcontext)
  {
    pp_free_context(ppcontext);
    ppcontext=NULL;
  }
  if(ppmode)
  {
    pp_free_mode(ppmode);
    ppmode=NULL;
  }
  
} 
//*************************************************************
void ADMVideoLavPPDeint::setup(void)
{
  char string[1024];
  uint32_t ppCaps=0;
                
  string[0]=0;
  
        cleanup();
#if (defined( ARCH_X86)  || defined(ARCH_X86_64))
#define ADD(x,y) if( CpuCaps::has##x()) ppCaps|=PP_CPU_CAPS_##y;                
                ADD(MMX,MMX);           
                ADD(3DNOW,3DNOW);
                ADD(MMXEXT,MMX2);
#endif             
        cleanup();
#undef ADD       
#define ADD(z)  { if(string[0]) strcat(string,","#z); else strcpy(string,#z);}        
               
        if(_param->autolevel) ADD(al);
        switch(_param->deintType)
        {
          case PP_BM_NONE:break;
          case PP_BM_LINEAR_BLEND: ADD(lb);break;
          case PP_BM_LINEAR_INTER: ADD(li);break;
          case PP_BM_CUBIC_INTER: ADD(ci);break;
          case PP_BM_MEDIAN_INTER: ADD(md);break;
          case PP_BM_FFMPEG_DEINT: ADD(fd);break;                             
        }        


        ppcontext=pp_get_context(_info.width, _info.height, ppCaps);           
        ppmode=pp_get_mode_by_name_and_quality(string,1);;
        
        ADM_assert(ppcontext);
        ADM_assert(ppmode);
  
} 
//*************************************************************
ADMVideoLavPPDeint::~ADMVideoLavPPDeint()
{
  cleanup();
  if(_uncompressed)
        delete _uncompressed;
  DELETE(_param);
  _uncompressed=NULL;
}

//*************************************************************
uint8_t ADMVideoLavPPDeint::getCoupledConf( CONFcouple **couples)
{
  
  *couples=new CONFcouple(2);
  CSET(deintType);    
  CSET(autolevel);
  
  return 1;
}
//*************************************************************
uint8_t ADMVideoLavPPDeint::getFrameNumberNoAlloc(uint32_t frame,
                                              uint32_t *len,
                                              ADMImage *data,
                                              uint32_t *flags)
{

 
  ADM_assert(frame<_info.nb_frames);
        // read uncompressed frame
  if(!_in->getFrameNumberNoAlloc(frame, len,_uncompressed,flags)) return 0;

  
  //
  uint8_t *iBuff[3],*oBuff[3];
  int strideTab[3],strideTab2[3];                 
                                                                
  oBuff[0]=YPLANE(data);
  oBuff[1]=UPLANE(data);
  oBuff[2]=VPLANE(data);
                                
  iBuff[0]=YPLANE(_uncompressed);
  iBuff[1]=UPLANE(_uncompressed);
  iBuff[2]=VPLANE(_uncompressed);
                                
                                
  strideTab[0]=strideTab2[0]=_info.width;
  strideTab[1]=strideTab2[1]=_info.width>>1;
  strideTab[2]=strideTab2[2]=_info.width>>1;
        
  int type;
  if(_uncompressed->flags&AVI_KEY_FRAME)
    type=1;
  else if(_uncompressed->flags & AVI_B_FRAME)
    type=3;
  else
    type=2;
  pp_postprocess(
        iBuff,
        strideTab,
        oBuff,
        strideTab2,
        _info.width,
        _info.height,
        NULL,
        0,
        ppmode,
        ppcontext,
        type); // I ?
                                
  
  data->copyInfo(_uncompressed);
  return 1;
}
